#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase

class TestFollowUpBot(BotTestCase):
    bot_name = "followup"

    def test_bot(self):
        expected_send_reply = {
            "": 'Please specify the message you want to send to followup stream after @mention-bot'
        }
        self.check_expected_responses(expected_send_reply, expected_method='send_reply')

        expected_send_message = {
            "foo": {
                'type': 'stream',
                'to': 'followup',
                'subject': 'foo_sender@zulip.com',
                'content': 'from foo_sender@zulip.com: foo',
            },
            "I have completed my task": {
                'type': 'stream',
                'to': 'followup',
                'subject': 'foo_sender@zulip.com',
                'content': 'from foo_sender@zulip.com: I have completed my task',
            },
        }
        self.check_expected_responses(expected_send_message, expected_method='send_message')



# See readme.md for instructions on running this code.
from __future__ import print_function
import logging
from six.moves.urllib import error
from six.moves.urllib.request import urlopen

# Uses the Google search engine bindings
#   pip install --upgrade google
from google import search


def get_google_result(search_keywords):
    help_message = "To use this bot, start messages with @mentioned-bot, \
                    followed by what you want to search for. If \
                    found, Zulip will return the first search result \
                    on Google.\
                    \
                    An example message that could be sent is:\
                    '@mentioned-bot zulip' or \
                    '@mentioned-bot how to create a chatbot'."
    if search_keywords == 'help':
        return help_message
    elif search_keywords == '' or search_keywords is None:
        return help_message
    else:
        try:
            urls = search(search_keywords, stop=20)
            urlopen('http://216.58.192.142', timeout=1)
        except error.URLError as er:
            logging.exception(er)
            return 'Error: No internet connection. {}.'.format(er)
        except Exception as e:
            logging.exception(e)
            return 'Error: Search failed. {}.'.format(e)

        try:
            url = next(urls)
        except AttributeError as a_err:
            # google.search query failed and urls is of object
            # 'NoneType'
            logging.exception(a_err)
            return "Error: Google search failed with a NoneType result. {}.".format(a_err)
        except TypeError as t_err:
            # google.search query failed and returned None
            # This technically should not happen but the prior
            # error check assumed this behavior
            logging.exception(t_err)
            return "Error: Google search function failed. {}.".format(t_err)
        except Exception as e:
            logging.exception(e)
            return 'Error: Search failed. {}.'.format(e)

        return 'Success: {}'.format(url)


class GoogleSearchHandler(object):
    '''
    This plugin allows users to enter a search
    term in Zulip and get the top URL sent back
    to the context (stream or private) in which
    it was called. It looks for messages starting
    with @mentioned-bot.
    '''

    def usage(self):
        return '''
            This plugin will allow users to search
            for a given search term on Google from
            Zulip. Use '@mentioned-bot help' to get
            more information on the bot usage. Users
            should preface messages with
            @mentioned-bot.
            '''

    def handle_message(self, message, bot_handler, state_handler):
        original_content = message['content']
        result = get_google_result(original_content)
        bot_handler.send_reply(message, result)

handler_class = GoogleSearchHandler


def test():
    try:
        urlopen('http://216.58.192.142', timeout=1)
        print('Success')
        return True
    except error.URLError as e:
        print('Error: {}'.format(e))
        return False

if __name__ == '__main__':
    test()



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase

class TestEncryptBot(BotTestCase):
    bot_name = "encrypt"

    def test_bot(self):
        expected = {
            "": "Encrypted/Decrypted text: ",
            "Let\'s Do It": "Encrypted/Decrypted text: Yrg\'f Qb Vg",
            "me&mom together..!!": "Encrypted/Decrypted text: zr&zbz gbtrgure..!!",
            "foo bar": "Encrypted/Decrypted text: sbb one",
            "Please encrypt this": "Encrypted/Decrypted text: Cyrnfr rapelcg guvf",
        }
        self.check_expected_responses(expected)



#!/usr/bin/env python
from __future__ import print_function
from __future__ import absolute_import

import argparse
import sys
import os
from types import ModuleType
from importlib import import_module
from os.path import basename, splitext

import six
from six.moves import configparser
import mock
from mock import MagicMock, patch

from zulip_bots.lib import run_message_handler_for_bot, StateHandler
from zulip_bots.provision import provision_bot
from zulip_bots.run import import_module_from_source, name_and_patch_match

def parse_args():
    usage = '''
        zulip-bot-output <bot_name> --message "Send this message to the bot"
        Example: zulip-bot-output xkcd --message "1"
        This tool can be used for testing bots by sending simple messages
        and capturing the response.
        (Internally, this program loads bot-related code from the
        library code and then feeds the message provided
        in the command to the library code to handle.)
        '''

    parser = argparse.ArgumentParser(usage=usage)
    parser.add_argument('name',
                        action='store',
                        nargs='?',
                        default=None,
                        help='the name of an existing bot to run')
    parser.add_argument('--message',
                        action='store',
                        help='the message content to send to the bot')

    parser.add_argument('--path-to-bot',
                        action='store',
                        help='path to the file with the bot handler class')

    parser.add_argument('--force',
                        action='store_true',
                        help='Try running the bot even if dependencies install fails.')

    parser.add_argument('--provision',
                        action='store_true',
                        help='Install dependencies for the bot.')

    options = parser.parse_args()

    if not options.name and not options.path_to_bot:
        error_message = """
You must either specify the name of an existing bot or
specify a path to the file (--path-to-bot) that contains
the bot handler class.
"""
        parser.error(error_message)
    # Checks if both name and path to bots are provided:
    # checks if both of these are in sync, otherwise we'll
    # have to be bias towards one and the user may get incorrect
    # result.
    elif not name_and_patch_match(options.name, options.path_to_bot):
        error_message = """
Please make sure that the given name of the bot and the
given path to the bot are same and valid.
"""
        parser.error(error_message)

    return options

def main():
    # type: () -> None
    options = parse_args()
    bot_name = options.name
    if options.path_to_bot:
        if options.provision:
            bot_dir = os.path.dirname(os.path.abspath(options.path_to_bot))
            provision_bot(bot_dir, options.force)
        lib_module = import_module_from_source(options.path_to_bot, name=bot_name)
    elif options.name:
        if options.provision:
            current_dir = os.path.dirname(os.path.abspath(__file__))
            bots_parent_dir = os.path.join(current_dir, "bots")
            bot_dir = os.path.join(bots_parent_dir, options.name)
            provision_bot(bot_dir, options.force)
        lib_module = import_module('zulip_bots.bots.{bot}.{bot}'.format(bot=bot_name))

    message = {'content': options.message, 'sender_email': 'foo_sender@zulip.com'}
    message_handler = lib_module.handler_class()

    with patch('zulip_bots.lib.ExternalBotHandler') as mock_bot_handler:
        def get_config_info(bot_name, section=None, optional=False):
            # type: (str, Optional[str], Optional[bool]) -> Dict[str, Any]
            conf_file_path = os.path.realpath(os.path.join(
                'zulip_bots', 'bots', bot_name, bot_name + '.conf'))
            section = section or bot_name
            config = configparser.ConfigParser()
            try:
                with open(conf_file_path) as conf:
                    config.readfp(conf)  # type: ignore
            except IOError:
                if optional:
                    return dict()
                raise
            return dict(config.items(section))

        mock_bot_handler.get_config_info = get_config_info
        if hasattr(message_handler, 'initialize') and callable(message_handler.initialize):
            message_handler.initialize(mock_bot_handler)

        mock_bot_handler.send_reply = MagicMock()
        mock_bot_handler.send_message = MagicMock()
        message_handler.handle_message(
            message=message,
            bot_handler=mock_bot_handler,
            state_handler=StateHandler()
        )
        print("On sending ", options.name, " bot the following message:\n\"", options.message, "\"")

        # send_reply and send_message have slightly arguments; the
        # following takes that into account.
        #   send_reply(original_message, response)
        #   send_message(response_message)
        if mock_bot_handler.send_reply.called:
            print("\nThe bot gives the following output message:\n\"", list(mock_bot_handler.send_reply.call_args)[0][1], "\"")
        elif mock_bot_handler.send_message.called:
            print("\nThe bot sends the following output to zulip:\n\"", list(mock_bot_handler.send_message.call_args)[0][0], "\"")
        else:
            print("\nThe bot sent no reply.")

if __name__ == '__main__':
    main()



from __future__ import print_function
from __future__ import absolute_import

import datetime as dt
import re
import requests
from six.moves import range

class FoursquareHandler(object):
    def initialize(self, bot_handler):
        self.api_key = bot_handler.get_config_info('foursquare', 'Foursquare')['api_key']

    def usage(self):
        return '''
            This plugin allows users to search for restaurants nearby an inputted
            location to a limit of 3 venues for every location. The name, address
            and description of the restaurant will be outputted.
            It looks for messages starting with '@mention-bot'.
            If you need help, simply type:
            @mention-bot /help into the Compose Message box

            Sample input:
            @mention-bot Chicago, IL
            @mention-bot help
            '''

    help_info = '''
The Foursquare bot can receive keyword limiters that specify the location, distance (meters) and
cusine of a restaurant in that exact order.
Please note the required use of quotes in the search location.

Example Inputs:
@mention-bot 'Millenium Park' 8000 donuts
@mention-bot 'Melbourne, Australia' 40000 seafood
                '''

    def format_json(self, venues):
        def format_venue(venue):
            name = venue['name']
            address = ', '.join(venue['location']['formattedAddress'])
            keyword = venue['categories'][0]['pluralName']
            blurb = '\n'.join([name, address, keyword])
            return blurb

        return '\n'.join(format_venue(venue) for venue in venues)

    def send_info(self, message, letter, bot_handler):
        bot_handler.send_reply(message, letter)

    def handle_message(self, message, bot_handler, state_handler):
        words = message['content'].split()
        if "/help" in words:
            self.send_info(message, self.help_info, bot_handler)
            return

        # These are required inputs for the HTTP request.
        try:
            params = {'limit': '3'}
            params['near']  = re.search('\'[A-Za-z]\w+[,]?[\s\w+]+?\'', message['content']).group(0)
            params['v'] = 20170108
            params['oauth_token'] = self.api_key
        except AttributeError:
            pass

        # Optional params for HTTP request.
        if len(words) >= 1:
            try:
                params['radius'] = re.search('([0-9]){3,}', message['content']).group(0)
            except AttributeError:
                pass
            try:
                params['query'] = re.search('\s([A-Za-z]+)$', message['content']).group(0)[1:]
            except AttributeError:
                params['query'] = 'food'

        response = requests.get('https://api.foursquare.com/v2/venues/search?',
                                params=params)
        print(response.url)
        if response.status_code == 200:
            received_json = response.json()
        else:
            self.send_info(message,
                           "Invalid Request\nIf stuck, try '@mention-bot help'.",
                           bot_handler)
            return

        if received_json['meta']['code'] == 200:
            response_msg = ('Food nearby ' + params['near'] +
                            ' coming right up:\n' +
                            self.format_json(received_json['response']['venues']))
            self.send_info(message, response_msg, bot_handler)
            return

        self.send_info(message,
                       "Invalid Request\nIf stuck, try '@mention-bot help'.",
                       bot_handler)
        return

handler_class = FoursquareHandler



#!/usr/bin/env python
from __future__ import print_function
from __future__ import absolute_import

import logging
import argparse
import sys
import os
from types import ModuleType
from importlib import import_module
from os.path import basename, splitext

import six

from zulip_bots.lib import run_message_handler_for_bot
from zulip_bots.provision import provision_bot

def import_module_from_source(path, name=None):
    if not name:
        name = splitext(basename(path))[0]

    if six.PY2:
        import imp
        module = imp.load_source(name, path)
    else:
        import importlib.util
        spec = importlib.util.spec_from_file_location(name, path)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)

    return module

def name_and_patch_match(given_name, path_to_bot):
    if given_name and path_to_bot:
        name_by_path = os.path.splitext(os.path.basename(path_to_bot))[0]
        if (given_name != name_by_path):
            return False
    return True

def parse_args():
    usage = '''
        zulip-run-bot <bot_name>
        Example: zulip-run-bot followup
        (This program loads bot-related code from the
        library code and then runs a message loop,
        feeding messages to the library code to handle.)
        Please make sure you have a current ~/.zuliprc
        file with the credentials you want to use for
        this bot.
        See lib/readme.md for more context.
        '''

    parser = argparse.ArgumentParser(usage=usage)
    parser.add_argument('name',
                        action='store',
                        nargs='?',
                        default=None,
                        help='the name of an existing bot to run')

    parser.add_argument('--quiet', '-q',
                        action='store_true',
                        help='Turn off logging output.')

    parser.add_argument('--config-file',
                        action='store',
                        help='(alternate config file to ~/.zuliprc)')

    parser.add_argument('--path-to-bot',
                        action='store',
                        help='path to the file with the bot handler class')

    parser.add_argument('--force',
                        action='store_true',
                        help='Try running the bot even if dependencies install fails.')

    parser.add_argument('--provision',
                        action='store_true',
                        help='Install dependencies for the bot.')
    options = parser.parse_args()

    if not options.name and not options.path_to_bot:
        error_message = """
You must either specify the name of an existing bot or
specify a path to the file (--path-to-bot) that contains
the bot handler class.
"""
        parser.error(error_message)
    # Checks if both name and path to bots are provided:
    # checks if both of these are in sync, otherwise we'll
    # have to be bias towards one and the user may get incorrect
    # result.
    elif not name_and_path_match(options.name, options.path_to_bot):
        error_message = """
Please make sure that the given name of the bot and the
given path to the bot are same and valid.
"""
        parser.error(error_message)

    return options


def main():
    # type: () -> None
    options = parse_args()
    bot_name = options.name
    if options.path_to_bot:
        if options.provision:
            bot_dir = os.path.dirname(os.path.abspath(options.path_to_bot))
            provision_bot(bot_dir, options.force)
        lib_module = import_module_from_source(options.path_to_bot, name=bot_name)
    elif options.name:
        if options.provision:
            current_dir = os.path.dirname(os.path.abspath(__file__))
            bots_parent_dir = os.path.join(current_dir, "bots")
            bot_dir = os.path.join(bots_parent_dir, options.name)
            provision_bot(bot_dir, options.force)
        lib_module = import_module('zulip_bots.bots.{bot}.{bot}'.format(bot=bot_name))

    if not options.quiet:
        logging.basicConfig(stream=sys.stdout, level=logging.INFO)

    run_message_handler_for_bot(
        lib_module=lib_module,
        config_file=options.config_file,
        quiet=options.quiet,
        bot_name=bot_name
    )

if __name__ == '__main__':
    main()



from __future__ import print_function

import datetime as dt
import requests

class CommuteHandler(object):
    '''
    This plugin provides information regarding commuting
    from an origin to a destination, providing a multitude of information.
    It looks for messages starting with @mention of the bot.
    '''

    def initialize(self, bot_handler):
        self.api_key = bot_handler.get_config_info('commute', 'Google.com')['api_key']

    def usage(self):
        return '''
            This plugin will allow briefings of estimated travel times,
            distances and fare information for transit travel.
            It can vary outputs depending on traffic conditions, departure and
            arrival times as well as user preferences
            (toll avoidance, preference for bus travel, etc.).
            It looks for messages starting with @mention of the bot.

            Users should input an origin and a destination
            in any stream or through private messages to the bot to receive a
            response in the same stream or through private messages if the
            input was originally private.

            Sample input:
            @mention-botname origins=Chicago,IL,USA destinations=New+York,NY,USA
            @mention-botname help
            '''

    help_info = '''
    Obligatory Inputs:
        Origin e.g. origins=New+York,NY,USA
        Destination e.g. destinations=Chicago,IL,USA
    Optional Inputs:
        Mode Of Transport (inputs: driving, walking, bicycling, transit)
        Default mode (no mode input) is driving
        e.g. mode=driving or mode=transit
        Units (metric or imperial)
        e.g. units=metric
        Restrictions (inputs: tolls, highways, ferries, indoor)
        e.g. avoid=tolls
        Departure Time (inputs: now or (YYYY, MM, DD, HH, MM, SS) departing)
        e.g. departure_time=now or departure_time=2016,12,17,23,40,40
        Arrival Time (inputs: (YYYY, MM, DD, HH, MM, SS) arriving)
        e.g. arrival_time=2016,12,17,23,40,40
        Languages:
        Languages list: https://developers.google.com/maps/faq#languagesupport)
        e.g. language=fr

    Sample request:
        @mention-botname origins=Chicago,IL,USA destinations=New+York,NY,USA language=fr

    Please note:
        Fare information can be derived, though is solely dependent on the
        availability of the information
        python run.py bots/followup/followup.py --config-file ~/.zuliprc-local
        released by public transport operators.
        Duration in traffic can only be derived if a departure time is set.
        If a location has spaces in its name, please use a + symbol in the
        place of the space/s.
        A departure time and a arrival time can not be inputted at the same time
        To add more than 1 input for a category,
        e.g. more than 1 destinations,
        use (|), e.g. destinations=Empire+State+Building|Statue+Of+Liberty
        No spaces within addresses.
        Departure times and arrival times must be in the UTC timezone,
        you can use the timezone bot.
                '''

    # determines if bot will respond as a private message/ stream message
    def send_info(self, message, letter, bot_handler):
        bot_handler.send_reply(message, letter)

    def calculate_seconds(self, time_str):
        times = time_str.split(',')
        times = [int(x) for x in times]
        # UNIX time from January 1, 1970 00:00:00
        unix_start_date = dt.datetime(1970, 1, 1, 0, 0, 0)
        requested_time = dt.datetime(*times)
        total_seconds = str(int((requested_time-unix_start_date)
                                .total_seconds()))
        return total_seconds

    # adds departure time and arrival time paramaters into HTTP request
    def add_time_to_params(self, params):
        # limited to UTC timezone because of difficulty with user inputting
        # correct string for input
        if 'departure_time' in params:
            if 'departure_time' != 'now':
                params['departure_time'] = self.calculate_seconds(params['departure_time'])
        elif 'arrival_time' in params:
            params['arrival_time'] = self.calculate_seconds(params['arrival_time'])
        return

    # gets content for output and sends it to user
    def get_send_content(self, rjson, params, message, bot_handler):
        try:
            # JSON list of output variables
            variable_list = rjson["rows"][0]["elements"][0]
            # determines if user has valid inputs
            not_found = (variable_list["status"] == "NOT_FOUND")
            invalid_request = (rjson["status"] == "INVALID_REQUEST")
            no_result = (variable_list["status"] == "ZERO_RESULTS")

            if no_result:
                self.send_info(message,
                               "Zero results\nIf stuck, try '@commute help'.",
                               bot_handler)
                return
            elif not_found or invalid_request:
                raise IndexError
        except IndexError:
            self.send_info(message,
                           "Invalid input, please see instructions."
                           "\nIf stuck, try '@commute help'.", bot_handler)
            return

        # origin and destination strings
        begin = 'From: ' + rjson["origin_addresses"][0]
        end = 'To: ' + rjson["destination_addresses"][0]
        distance = 'Distance: ' + variable_list["distance"]["text"]
        duration = 'Duration: ' + variable_list["duration"]["text"]
        output = begin + '\n' + end + '\n' + distance
        # if user doesn't know that default mode is driving
        if 'mode' not in params:
            mode = 'Mode of Transport: Driving'
            output += '\n' + mode

        # determines if fare information is available
        try:
            fare = ('Fare: ' + variable_list["fare"]["currency"] +
                    variable_list["fare"]["text"])
            output += '\n' + fare
        except (KeyError, IndexError):
            pass

        # determines if traffic duration information is available
        try:
            traffic_duration = ('Duration in traffic: ' +
                                variable_list["duration_in_traffic"]
                                ["text"])
            output += '\n' + traffic_duration
        except (KeyError, IndexError):
            output += '\n' + duration

        # bot sends commute information to user
        self.send_info(message, output, bot_handler)

    # creates parameters for HTTP request
    def parse_pair(self, content_list):
        result = {}
        for item in content_list:
            # enables key value pair
            org = item.split('=')
            # ensures that invalid inputs are not entered into url request
            if len(org) != 2:
                continue
            key, value = org
            result[key] = value
        return result

    def receive_response(self, params, message, bot_handler):
        def validate_requests(request):
            if request.status_code == 200:
                return request.json()
            else:
                self.send_info(message,
                               "Something went wrong. Please try again." +
                               " Error: {error_num}.\n{error_text}"
                               .format(error_num=request.status_code,
                                       error_text=request.text), bot_handler)
                return
        r = requests.get('https://maps.googleapis.com/maps/api/' +
                         'distancematrix/json', params=params)
        result = validate_requests(r)
        return result

    def handle_message(self, message, bot_handler, state_handler):
        original_content = message['content']
        query = original_content.split()

        if "help" in query:
            self.send_info(message, self.help_info, bot_handler)
            return

        params = self.parse_pair(query)
        params['key'] = self.api_key
        self.add_time_to_params(params)

        rjson = self.receive_response(params, message, bot_handler)
        if not rjson:
            return

        self.get_send_content(rjson, params, message, bot_handler)

handler_class = CommuteHandler
handler = CommuteHandler()

def test_parse_pair():
    result = handler.parse_pair(['departure_time=2016,12,20,23,59,00',
                                'dog_foo=cat-foo'])
    assert result == dict(departure_time='2016,12,20,23,59,00',
                          dog_foo='cat-foo')

def test_calculate_seconds():
    result = handler.calculate_seconds('2016,12,20,23,59,00')
    assert result == str(1482278340)

def test_helper_functions():
    test_parse_pair()
    test_calculate_seconds()

if __name__ == '__main__':
    test_helper_functions()
    print('Success')



# See readme.md for instructions on running this code.
from __future__ import absolute_import
from __future__ import print_function
from . import github
import json
import logging
import os
import requests


class InputError(IndexError):
    '''raise this when there is an error with the information the user has entered'''


class GitHubHandler(object):
    '''
    This plugin allows you to comment on a GitHub issue, under a certain repository.
    It looks for messages starting with '@mention-bot'.
    '''

    def usage(self):
        return '''
            This bot will allow users to comment on a GitHub issue.
            Users should preface messages with '@mention-bot'.
            You will need to have a GitHub account.

            Before running this, make sure to get a GitHub OAuth token.
            The token will need to be authorized for the following scopes:
            'gist, public_repo, user'.
            Store it in the '~/.github_auth.conf' file, along with your username, in the format:
            github_repo = <repo_name>  (The name of the repo to post to)
            github_repo_owner = <repo_owner>  (The owner of the repo to post to)
            github_username = <username>  (The username of the GitHub bot)
            github_token = <oauth_token>   (The personal access token for the GitHub bot)

            Leave the first two options blank.

            Please use this format in your message to the bot:
            '<repository_owner>/<repository>/<issue_number>/<your_comment>'.
            '''

    def handle_message(self, message, bot_handler, state_handler):
        original_content = message['content']
        original_sender = message['sender_email']

        handle_input(bot_handler, original_content, original_sender)

handler_class = GitHubHandler


def send_to_github(repo_owner, repo, issue, comment_body):
    session = github.auth()
    comment = {
        'body': comment_body
    }
    r = session.post('https://api.github.com/repos/%s/%s/issues/%s/comments' % (repo_owner, repo, issue),
                     json.dumps(comment))

    return r.status_code


def get_values_message(original_content):
    # gets rid of whitespace around the edges, so that they aren't a problem in the future
    message_content = original_content.strip()
    # splits message by '/' which will work if the information was entered correctly
    message_content = message_content.split('/')
    try:
        # this will work if the information was entered correctly
        user = github.get_username()
        repo_owner = message_content[2]
        repo = message_content[3]
        issue = message_content[4]
        comment_body = message_content[5]

        return dict(user=user, repo_owner=repo_owner, repo=repo, issue=issue, comment_body=comment_body)
    except IndexError:
        raise InputError


def handle_input(bot_handler, original_content, original_sender):
    try:
        params = get_values_message(original_content)

        status_code = send_to_github(params['repo_owner'], params['repo'],
                                     params['issue'], params['comment_body'])

        if status_code == 201:
            # sending info to github was successful!
            reply_message = "You commented on issue number " + params['issue'] + " under " + \
                            params['repo_owner'] + "'s repository " + params['repo'] + "!"

            send_message(bot_handler, reply_message, original_sender)

        elif status_code == 404:
            # this error could be from an error with the OAuth token
            reply_message = "Error code: " + str(status_code) + " :( There was a problem commenting on issue number " \
                            + params['issue'] + " under " + \
                            params['repo_owner'] + "'s repository " + params['repo'] + \
                            ". Do you have the right OAuth token?"

            send_message(bot_handler, reply_message, original_sender)

        else:
            # sending info to github did not work
            reply_message = "Error code: " + str(status_code) +\
                            " :( There was a problem commenting on issue number " \
                            + params['issue'] + " under " + \
                            params['repo_owner'] + "'s repository " + params['repo'] + \
                            ". Did you enter the information in the correct format?"

            send_message(bot_handler, reply_message, original_sender)
    except InputError:
            message = "It doesn't look like the information was entered in the correct format." \
                      " Did you input it like this? " \
                      "'/<username>/<repository_owner>/<repository>/<issue_number>/<your_comment>'."
            send_message(bot_handler, message, original_sender)
            logging.error('there was an error with the information you entered')


def send_message(bot_handler, message, original_sender):
    # function for sending a message
    bot_handler.send_message(dict(
        type='private',
        to=original_sender,
        content=message,
    ))



# See readme.md for instructions on running this bot.
import requests
from bs4 import BeautifulSoup

class YoutubeHandler(object):
    def usage(self):
        return '''
            This bot will return the first Youtube search result for the give query.
            '''

    def handle_message(self, message, bot_handler, state_handler):
        help_content = '''
            To use the, Youtube Bot send `@mention-bot search terms`
            Example:
            @mention-bot funny cats
            '''.strip()
        if message['content'] == '':
            bot_handler.send_reply(message, help_content)
        else:
            text_to_search = message['content']
            url = "https://www.youtube.com/results?search_query=" + text_to_search
            r = requests.get(url)
            soup = BeautifulSoup(r.text, 'lxml')
            video_id = soup.find(attrs={'class': 'yt-uix-tile-link'})
            try:
                link = 'https://www.youtube.com' + video_id['href']
                bot_handler.send_reply(message, link)
            except TypeError:
                bot_handler.send_reply(message, 'No video found for specified search terms')

handler_class = YoutubeHandler



"""
This bot uses the python library `howdoi` which is not a
dependency of Zulip. To use this module, you will have to
install it in your local machine. In your terminal, enter
the following command:

    $ sudo pip install howdoi --upgrade

Note:
    * You might have to use `pip3` if you are using python 3.
    * The install command would also download any dependency
      required by `howdoi`.
"""

from __future__ import absolute_import

import sys
import logging
from textwrap import fill
try:
    from howdoi.howdoi import howdoi
except ImportError:
    logging.error("Dependency missing!!\n%s" % (__doc__))
    sys.exit(0)


class HowdoiHandler(object):
    '''
    This plugin facilitates searching Stack Overflow for
    techanical answers based on the Python library `howdoi`.
    To get the best possible answer, only include keywords
    in your questions.

    There are two possible commands:
    * @mention-bot howdowe > This would return the answer to the same
                 stream that it was called from.

    * @mention-bot howdoi > The bot would send a private message to the
                user containing the answer.

    By default, howdoi only returns the coding section of the
    first search result if possible, to see the full answer
    from Stack Overflow, append a '!' to the commands.
    (ie '@mention-bot howdoi!', '@mention-bot howdowe!')
    '''

    MAX_LINE_LENGTH = 85

    def usage(self):
        return '''
            This plugin will allow users to get techanical
            answers from Stackoverflow. Users should preface
            their questions with one of the following:

                * @mention-bot howdowe > Answer to the same stream
                * @mention-bot howdoi > Answer via private message

                * @mention-bot howdowe! OR @mention-bot howdoi! > Full answer from SO
            '''

    def line_wrap(self, string, length):
        lines = string.split("\n")

        wrapped = [(fill(line) if len(line) > length else line)
                   for line in lines]

        return "\n".join(wrapped).strip()

    def get_answer(self, command, query):
        question = query[len(command):].strip()
        result = howdoi(dict(
            query=question,
            num_answers=1,
            pos=1,
            all=command[-1] == '!',
            color=False
        ))
        _answer = self.line_wrap(result, HowdoiHandler.MAX_LINE_LENGTH)

        answer = "Answer to '%s':\n```\n%s\n```" % (question, _answer)

        return answer

    def handle_message(self, message, bot_handler, state_handler):
        question = message['content'].strip()

        if question.startswith('howdowe!'):
            bot_handler.send_message(dict(
                type='stream',
                to=message['display_recipient'],
                subject=message['subject'],
                content=self.get_answer('howdowe!', question)
            ))

        elif question.startswith('howdoi!'):
            bot_handler.send_message(dict(
                type='private',
                to=message['sender_email'],
                content=self.get_answer('howdoi!', question)
            ))

        elif question.startswith('howdowe'):
            bot_handler.send_message(dict(
                type='stream',
                to=message['display_recipient'],
                subject=message['subject'],
                content=self.get_answer('howdowe', question)
            ))

        elif question.startswith('howdoi'):
            bot_handler.send_message(dict(
                type='private',
                to=message['sender_email'],
                content=self.get_answer('howdoi', question)
            ))


handler_class = HowdoiHandler



#!/usr/bin/env python

# The purpose of this file is to handle all requests
# to the Github API, which will make sure that all
# requests are to the same account, and that all requests
# authenticated correctly and securely
# The sole purpose of this is to authenticate, and it will
# return the requests session that is properly authenticated
import logging
import os
import requests
import six.moves.configparser


# This file contains the oauth token for a github user, their username, and optionally, a repository
# name and owner. All in the format:
# github_repo
# github_repo_owner
# github_username
# github_token
CONFIG_FILE = '~/.github-auth.conf'

global config
config = six.moves.configparser.ConfigParser()  # Sets up the configParser to read the .conf file
config.read([os.path.expanduser(CONFIG_FILE)])  # Reads the config file


def auth():
    # Creates and authorises a requests session
    session = requests.session()
    session.auth = (get_username(), get_oauth_token())

    return session


def get_oauth_token():
    _get_data('token')


def get_username():
    _get_data('username')


def get_repo():
    _get_data('repo')


def get_repo_owner():
    _get_data('repo_owner')


def _get_data(key):
    try:
        return config.get('github', 'github_%s' % (key))
    except Exception:
        logging.exception('GitHub %s not supplied in ~/.github-auth.conf.' % (key))



from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()
from . import github
import json
import os
import requests
import six.moves.configparser
import urllib.error
import urllib.parse
import urllib.request

class IssueHandler(object):
    '''
    This plugin facilitates sending issues to github, when
    an item is prefixed with '@mention-bot'.

    It will also write items to the issues stream, as well
    as reporting it to github
    '''

    URL = 'https://api.github.com/repos/{}/{}/issues'
    CHARACTER_LIMIT = 70
    CONFIG_FILE = '~/.github-auth.conf'

    def __init__(self):
        self.repo_name = github.get_repo()
        self.repo_owner = github.get_repo_owner()

    def usage(self):
        return '''
            This plugin will allow users to flag messages
            as being issues with Zulip by using te prefix '@mention-bot'.

            Before running this, make sure to create a stream
            called "issues" that your API user can send to.

            Also, make sure that the credentials of the github bot have
            been typed in correctly, that there is a personal access token
            with access to public repositories ONLY,
            and that the repository name is entered correctly.

            Check ~/.github-auth.conf, and make sure there are
            github_repo = <repo_name>  (The name of the repo to post to)
            github_repo_owner = <repo_owner>  (The owner of the repo to post to)
            github_username = <username>  (The username of the GitHub bot)
            github_token = <oauth_token>   (The personal access token for the GitHub bot)
            '''

    def handle_message(self, message, bot_handler, state_handler):

        original_content = message['content']
        original_sender = message['sender_email']

        temp_content = 'by {}:'.format(original_sender,)
        new_content = temp_content + original_content
        # gets the repo url
        url_new = self.URL.format(self.REPO_OWNER, self.REPO_NAME)

        # signs into github using the provided username and password
        session = github.auth()

        issue_title = message['content'].strip()
        issue_content = ''
        new_issue_title = ''
        for part_of_title in issue_title.split():
            if len(new_issue_title) < self.CHARACTER_LIMIT:
                new_issue_title += '{} '.format(part_of_title)
            else:
                issue_content += '{} '.format(part_of_title)

        new_issue_title = new_issue_title.strip()
        issue_content = issue_content.strip()
        new_issue_title += '...'

        # Creates the issue json, that is transmitted to the github api servers
        issue = {
            'title': new_issue_title,
            'body': '{} **Sent by [{}](https://chat.zulip.org/#) from zulip**'.format(issue_content, original_sender),
            'assignee': '',
            'milestone': 'none',
            'labels': [''],
        }
        # Sends the HTTP post request
        r = session.post(url_new, json.dumps(issue))

        if r.ok:
            # sends the message onto the 'issues' stream so it can be seen by zulip users
            bot_handler.send_message(dict(
                type='stream',
                to='issues',
                subject=message['sender_email'],
                # Adds a check mark so that the user can verify if it has been sent
                content='{} :heavy_check_mark:'.format(new_content),
            ))
            return
            # This means that the issue has not been sent
            # sends the message onto the 'issues' stream so it can be seen by zulip users
            bot_handler.send_message(dict(
                type='stream',
                to='issues',
                subject=message['sender_email'],
                # Adds a cross so that the user can see that it has failed, and provides a link to a
                # google search that can (hopefully) direct them to the error
                content='{} :x: Code: [{}](https://www.google.com/search?q=Github HTTP {} Error {})'
                        .format(new_content, r.status_code, r.status_code, r.content),
            ))

handler_class = IssueHandler



# See readme.md for instructions on running this code.


class IncrementorHandler(object):

    def usage(self):
        return '''
        This is a boilerplate bot that makes use of the
        update_message function. For the first @-mention, it initially
        replies with one message containing a `1`. Every time the bot
        is @-mentioned, this number will be incremented in the same message.
        '''

    def handle_message(self, message, bot_handler, state_handler):
        with state_handler.state({'number': 0, 'message_id': None}) as state:
            state['number'] += 1
            if state['message_id'] is None:
                result = bot_handler.send_reply(message, str(state['number']))
                state['message_id'] = result['id']
            else:
                bot_handler.update_message(dict(
                    message_id = state['message_id'],
                    content = str(state['number'])
                ))


handler_class = IncrementorHandler



import json
import os

from random import choice

try:
    from chatterbot import ChatBot
    from chatterbot.trainers import ChatterBotCorpusTrainer, ListTrainer
except ImportError:
    raise ImportError("""It looks like you are missing chatterbot.
                      Please: pip install chatterbot""")

BOTS_DIR = os.path.dirname(os.path.abspath(__file__))
DATABASE_PATH = os.path.join(BOTS_DIR, 'assets/var/database.db')
DIRECTORY_PATH = os.path.join(BOTS_DIR, 'assets')
VAR_PATH = os.path.join(BOTS_DIR, 'assets/var')
JOKES_PATH = 'assets/var/jokes.json'

if not os.path.exists(DIRECTORY_PATH):
    os.makedirs(DIRECTORY_PATH)

if not os.path.exists(VAR_PATH):
    os.makedirs(VAR_PATH)

# Create a new instance of a ChatBot
def create_chat_bot(no_learn):
    return ChatBot("John",
                   storage_adapter="chatterbot.storage.JsonFileStorageAdapter",
                   logic_adapters=
                   [
                       "chatterbot.logic.MathematicalEvaluation",
                       {
                           "import_path": "chatterbot.logic.BestMatch",
                           "response_selection_method": "chatterbot.response_selection.get_random_response",
                           "statement_comparison_function": "chatterbot.comparisons.levenshtein_distance"
                       }],
                   output_adapter="chatterbot.output.OutputAdapter",
                   output_format='text',
                   database=DATABASE_PATH,
                   silence_performance_warning="True",
                   read_only=no_learn)


class JohnHandler(object):
    '''
    This bot aims to be Zulip's virtual assistant. It
    finds the best match from a certain input.
    Also understands the English language and can
    mantain a conversation, joke and give useful information.
    '''

    def usage(self):
        return '''
            This bot aims to be Zulip's virtual assistant. It
            finds the best match from a certain input.
            Also understands the English language and can
            mantain a conversation, joke and give useful information.
            '''

    def initialize(self, bot_handler):
        self.bot = create_chat_bot(False)
        self.bot.set_trainer(ListTrainer)
        self.bot.train([
            "I want to contribute",
            """Contributors are more than welcomed! Please read
            https://github.com/zulip/zulip#how-to-get-involved-with-contributing-to-zulip
            to learn how to contribute.""",
        ])
        self.bot.train([
            "What is Zulip?",
            """Zulip is a powerful, open source group chat application. Written in Python
            and using the Django framework, Zulip supports both private messaging and group
            chats via conversation streams. You can learn more about the product and its
            features at https://www.zulip.org.""",
        ])
        self.bot.train([
            "I would like to request a remote dev instance",
            """Greetings! You should receive a response from one of our mentors soon.
            In the meantime, why don't you learn more about running Zulip on a development
            environment? https://zulip.readthedocs.io/en/latest/using-dev-environment.html""",
        ])
        self.bot.train([
            "Joke!",
            "Only if you ask nicely!",
        ])
        self.bot.train([
            "What is your name?",
            "I am John, my job is to assist you with Zulip.",
        ])
        self.bot.train([
            "What can you do?",
            "I can provide useful information and jokes if you follow etiquette.",
        ])
        with bot_handler.open(JOKES_PATH) as data_file:
            for joke in json.load(data_file):
                self.bot.train([
                    "Please can you tell me a joke?",
                    joke['joke'],
                ])
        self.bot.set_trainer(ChatterBotCorpusTrainer)
        self.bot.train(
            "chatterbot.corpus.english"
        )
        self.chatterbot = create_chat_bot(True)

    def handle_message(self, message, bot_handler, state_handler):
        original_content = message['content']
        bot_response = str(self.chatterbot.get_response(original_content))
        bot_handler.send_reply(message, bot_response)

handler_class = JohnHandler



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import print_function

from zulip_bots.test_lib import BotTestCase

class TestConverterBot(BotTestCase):
    bot_name = "converter"

    def test_bot(self):
        expected = {
            "": ('Too few arguments given. Enter `@convert help` '
                 'for help on using the converter.\n'),
            "foo bar": ('Too few arguments given. Enter `@convert help` '
                        'for help on using the converter.\n'),
            "2 m cm": "2.0 m = 200.0 cm\n",
            "12.0 celsius fahrenheit": "12.0 celsius = 53.600054 fahrenheit\n",
            "0.002 kilometer millimile": "0.002 kilometer = 1.2427424 millimile\n",
            "3 megabyte kilobit": "3.0 megabyte = 24576.0 kilobit\n",
        }
        self.check_expected_responses(expected)
